/*************************************************************************************************************************************************
 * ESP32cam takes photo
 * Sends to PC over serial line
 *************************************************************************************************************************************************/

#include "esp_camera.h"
#include "FS.h"
#include "SPI.h"
#include "driver/rtc_io.h"

#define CAMERA_MODEL_AI_THINKER

#include "camera_pins.h"

//#define ID_ADDRESS            0x00
//#define COUNT_ADDRESS         0x01
//#define ID_BYTE               0xAA
//
//#define TIME_TO_SLEEP  10            //time ESP32 will go to sleep (in seconds)
//#define uS_TO_S_FACTOR 1000000ULL   //conversion factor for micro seconds to seconds */

uint16_t nextImageNumber = 0;

esp_err_t initialise_camera() {
  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;

//  config.pixel_format = PIXFORMAT_RGB565;  
  config.pixel_format = PIXFORMAT_GRAYSCALE;  
  config.frame_size = FRAMESIZE_QVGA;
//  config.frame_size = FRAMESIZE_QQVGA;
  config.jpeg_quality = 10;
  config.fb_count = 1;

#if defined(CAMERA_MODEL_ESP_EYE)
  pinMode(13, INPUT_PULLUP);
  pinMode(14, INPUT_PULLUP);
#endif

  //initialize camera
  return esp_camera_init(&config);
}

void take_photo() {
  pinMode(4, OUTPUT);              //LED flash
//  digitalWrite(4, HIGH);            //flash LED ON 
  //take new image
  camera_fb_t * fb = NULL;
  //take photo
  fb = esp_camera_fb_get();
  if (!fb) 
  {
    Serial.println("photo failed");
    while(1);   //stop
  }
  digitalWrite(4, LOW);            //flash LED OFF

  Serial.print("\n\n\nImage ");
  Serial.print(fb->width); Serial.print(" ");
  Serial.print(fb->height); Serial.print(" ");
  Serial.print(fb->len); Serial.print(" ");
  Serial.print(fb->format); Serial.print(":");

  uint8_t *p = fb->buf;
  for (int i = 0; i < fb->len; i++) {
    Serial.write(*p); 
    p++;
  }

  esp_camera_fb_return(fb);
  fb = NULL;
}

void camera_settings() {
  sensor_t * s = esp_camera_sensor_get();

  s->set_gain_ctrl(s, 0); // auto gain off (1 or 0)
  s->set_exposure_ctrl(s, 1); // auto exposure  (1 or 0)
  s->set_agc_gain(s, 0); // set gain manually (0 - 30)
  s->set_aec_value(s, 300); // set exposure manually (0-1200)
  s->set_contrast(s, -2); // (-2 to 2)

//  s->set_brightness(s, -2);     // -2 to 2
  
  // s->set_brightness(s, 0); // (-2 to 2) - set brightness
  // s->set_awb_gain(s, 0); // Auto White Balance?
  // s->set_lenc(s, 0); // lens correction? (1 or 0)
  // s->set_raw_gma(s, 1); // (1 or 0)?
  // s->set_quality(s, 10); // (0 - 63)
  // s->set_whitebal(s, 1); // white balance
  // s->set_wb_mode(s, 1); // white balance mode (0 to 4)
  // s->set_aec2(s, 0); // automatic exposure sensor? (0 or 1)
  // s->set_aec_value(s, 0); // automatic exposure correction? (0-1200)
  // s->set_saturation(s, 0); // (-2 to 2)
  // s->set_hmirror(s, 0); // (0 or 1) flip horizontally
  // s->set_gainceiling(s, GAINCEILING_32X); // Image gain (GAINCEILING_x2, x4, x8, x16, x32, x64 or x128)
  // s->set_sharpness(s, 0); // (-2 to 2)
  // s->set_colorbar(s, 0); // (0 or 1) - testcard
  // s->set_special_effect(s, 0);
  // s->set_ae_level(s, 0); // auto exposure levels (-2 to 2)
  // s->set_bpc(s, 0); // black pixel correction
  // s->set_wpc(s, 0); // white pixel correction
  // s->set_dcw(s, 1); // downsize enable? (1 or 0)?
}

void setup() 
{
  Serial.begin(115200);
  Serial.println();
  Serial.println("Booting...");

  pinMode(4, INPUT);              //GPIO for LED flash
  digitalWrite(4, LOW);
  rtc_gpio_hold_dis(GPIO_NUM_4);  //diable pin hold if it was enabled before sleeping

  /**** this code magically makes the camera initialisation work reliably ****/
  /**** it turns the camera off and on ****/
  digitalWrite(PWDN_GPIO_NUM, LOW);
  delay(10);
  digitalWrite(PWDN_GPIO_NUM, HIGH);
  delay(10);

  esp_err_t err = initialise_camera();
  
  if (err != ESP_OK) 
  {
    Serial.printf("Camera init failed: 0x%x", err);
    while(1);   //stop
  }

  //delay(1000);

  camera_settings();

//  take_photo();
//
//  go_to_sleep();
}

void loop() 
{
  take_photo();
  delay(1000);
}
